#This code reproduces the analyses reported in Lewis et al (2021, Frontiers in Psychology).

#===========================
#SET DIRECTORY AND LOAD LIBRARIES AND FUNCTIONS
#===========================

#set directory in which data are located
setwd("~/Dropbox (The University of Manchester)/Frontiers/FINAL FILES/20210303/Cleaned Files/Final/Final")

#create or load functions we will use
colMax <- function(data) sapply(data, max, na.rm = TRUE)
source("~/Dropbox (The University of Manchester)/Frontiers/FINAL FILES/20210303/Cleaned Files/Final/Final/get.linearity.R")
library(vegan)
library(ggfortify)
library(lme4)
library(lmerTest)
library(kinship2)
library(coxme)
library(ggplot2)
library(mclust)
library(scatterplot3d)
library(AICcmodavg)
library(warbleR)

#===========================
#IMPORT AND PRE-PROCESS DATA
#===========================

#import manual note classifications
manual.note.classes=read.csv("Lewisetal2021_UnitTable2.csv",stringsAsFactors=FALSE,fill=TRUE)
#import spectral data on individual notes
spectral.note.data=read.csv("Lewisetal2021_AcousticParameters.csv",header=TRUE)
#import metadata on individual birds
meta.data=read.csv("Lewisetal2021_metadata.csv",header=TRUE)
meta.data=meta.data[order(meta.data$Bird.ID),]

#collect only the columns we will use from the spectral note data
col2use=c(1:9,12,13)
nxn.dat=spectral.note.data[,col2use]

#add Bird IDs to the spectral data
nxn.dat$Bird.ID=substr(nxn.dat[,2],1,6)
#assign note types to the note data
nxn.lab=paste(nxn.dat$sound.files,nxn.dat$selec)
nd.lab=paste(manual.note.classes$sound.files,manual.note.classes$selec)
nxn.dat$note.class=manual.note.classes$note_label[match(nxn.lab,nd.lab)]

#===========================
#CONDUCT PRELIMINARY DATA TRANSFORMATIONS
#===========================

#save the untransformed measures in case we want to use them later
nxn.dat.raw=nxn.dat

#Duration is right skewed, but can be approximately normalised by logging. Furthermore, theory suggests that logging standardises error for many measures of duration.
nxn.dat$duration=log(nxn.dat$duration)
colnames(nxn.dat)[4]="log.duration"

#time median and time IQR are better studied as proportions of duration
nxn.dat$time.median=nxn.dat$time.median/nxn.dat.raw$duration
nxn.dat$time.IQR=nxn.dat$time.IQR/nxn.dat.raw$duration

#Like durations, theory suggests that it is often sensible to log frequencies, because pitch changes with the log of frequency. Modulation index is slightly different, in that it is not measure of pitch, but it is strongly right-skewed and logging reduces that somewhat.
nxn.dat$meandom=log(nxn.dat$meandom)
colnames(nxn.dat)[7]="log.meandom"
nxn.dat$maxdom=log(nxn.dat$maxdom)
colnames(nxn.dat)[8]="log.maxdom"
nxn.dat$modindx=log(log(nxn.dat$modindx))
#Winsorise notes with no detected modulation to the lowest detectable modulation in the data set
nxn.dat$modindx[which(nxn.dat$modindx==-Inf)]=min(nxn.dat$modindx[which(nxn.dat$modindx!=-Inf)])
colnames(nxn.dat)[9]="log.log.modindx"
nxn.dat$meanpeakf=log(nxn.dat$meanpeakf)
colnames(nxn.dat)[11]="log.meanpeakf"

#Slope confounds change in frequency with duration, and has high error when the duration is short. So, we will convert slope to change in frequency, and work with that instead
nxn.dat$dfslope=nxn.dat$dfslope*nxn.dat.raw$duration
colnames(nxn.dat)[10]="fchange"

#===========================
#CREATE NOTE SUB-TYPES
#===========================

#We manually classified notes into broad categories as described in the methods. However, if we plot the notes in a given broad category produced by individual birds, there are often distinct clusters. This suggests that each broad note type may consist of multiple sub-types. In general, it is not possible to determine whether subtypes produced by different birds are different sub-types, or the same sub-type produced differently. We can easily see this by plotting a few notes as an example. In the figure produced below, the points illustrate notes of the type "Slope" produced by three different birds (color-coded). We can see that there are different cluster both among and within birds. Moreover, clusters from different birds can be partially overlapping, and we do not know whether these are different notes or the same note produced differently.

#look at representative clusters
colors=c("#999999","#E69F00","#56B4E9")
work.dat=nxn.dat[which((nxn.dat$Bird.ID=="JS0199"|nxn.dat$Bird.ID=="JS0264"|nxn.dat$Bird.ID=="JS0072")&nxn.dat$note.class=="Slope"),]
scatterplot3d(work.dat$log.duration,work.dat$log.meandom,work.dat$fchange,color=colors[as.numeric(as.factor(work.dat$Bird.ID))],xlab="duration",ylab="mean dominant frequency",zlab="change")

#With the above in mind, we will use Gaussian Mixture Models to assign notes of each broad note class to sub-types within birds. We will use only duration, mean dominant frequency, and change to define mixtures, because these measures are less error-prone than some of the others. Going forward, we will study these note sub-types at the within-bird level only.

#add the subtype column to nxn.dat
nxn.dat$sub.type=NA

#set birds and note classes to cycle through for classification
bird.list=unique(nxn.dat$Bird.ID)
note.list=unique(nxn.dat$note.class)

for (i in (1:length(bird.list))){
  for (j in (1:length(note.list))){
    
    #get the specific bird and note to work with
    work.dat=nxn.dat[which(nxn.dat$Bird.ID==bird.list[i]&nxn.dat$note.class==note.list[j]),]
    
    #
    if (nrow(work.dat)>0){
      
      #collect just the data we want to use
      to.clust=work.dat[,c(4,7,10)]
      
      #fit the GMM. If a bird produced a note type fewer than 5 times, we will force them all into one class. This is probably fine because the notes are unlikely to appear in the same song (which means they will not affect the structural analysis) and we won't use rare notes in the consistency analysis later. We will also force Chips into the same class, because the clustering algorithm creates an implausible number of Chip subclasses.
      if (nrow(work.dat)<5 | work.dat$note.class[1]=="Chip"){ 
        c.bic=NULL
        c.bic$classification=rep(1,nrow(work.dat))
      }else{
        c.bic=Mclust(to.clust,mod="VVV")
      }
      
      #save the sub-types to nxn.dat
      nxn.dat$sub.type[which(nxn.dat$Bird.ID==bird.list[i]&nxn.dat$note.class==note.list[j])]=c.bic$classification

    }
  }
}

#===========================
#PERPARE DATA FOR COMPUTING SONG STRUCTURAL FEATURES
#===========================

#To compute structural features, we will need daat about our songs in two different formats. First, we will need matrices where the columns are note types and the rows are songs. We will call these note.count.broad and note.count.fine for the manually defined note types and the computationally defined note sub-types, respectively. We will use these matrices to compute the Shannon's and Simpson's indices for note diversity in each song. Second, to compute song complexities, we will need a list of lists, where each list is the order of notes in a particular song. 

#compute note counts by song
nxs.dat=aggregate(rep(1,nrow(nxn.dat)),by=list(nxn.dat$note.class,nxn.dat$Bird.ID,nxn.dat$sound.files),sum)
for (i in 1:length(note.list)){
  nti.count=aggregate((nxs.dat$Group.1==note.list[i])*nxs.dat$x,by=list(nxs.dat$Group.2,nxs.dat$Group.3),sum)
  if (i==1){
    note.count.broad=nti.count
    colnames(note.count.broad)=c("Bird.ID","song.file",note.list[i])
  }else{
    note.count.broad=cbind(note.count.broad,nti.count$x)
    colnames(note.count.broad)[ncol(note.count.broad)]=note.list[i]
  }
}

#We can do the same thing for the fine-scale note counts. In this table, note subtypes with the same number but produced by different birds will be stored in the same column, even though we don't know that these are the same sub-type. This doesn't matter, because we will only study these note sub-types within birds.
note.list.fine=unique(paste(nxn.dat$note.class,nxn.dat$sub.typ,sep=""))
nxs.dat.fine=aggregate(rep(1,nrow(nxn.dat)),by=list(paste(nxn.dat$note.class,nxn.dat$sub.typ,sep=""),nxn.dat$Bird.ID,nxn.dat$sound.files),sum)
for (i in 1:length(note.list.fine)){
  nti.count=aggregate((nxs.dat.fine$Group.1==note.list.fine[i])*nxs.dat.fine$x,by=list(nxs.dat.fine$Group.2,nxs.dat.fine$Group.3),sum)
  if (i==1){
    note.count.fine=nti.count
    colnames(note.count.fine)=c("Bird.ID","song.file",note.list.fine[i])
  }else{
    note.count.fine=cbind(note.count.fine,nti.count$x)
    colnames(note.count.fine)[ncol(note.count.fine)]=note.list.fine[i]
  }
}

#create the list of lists for broad and fine scale notes
file.list=unique(nxn.dat$sound.files)
song.list.broad=list()
song.list.fine=list()
for (i in 1:length(file.list)){
  new.song=list(nxn.dat$note.class[which(nxn.dat$sound.files==file.list[i])])
  song.list.broad=append(song.list.broad,new.song)
  new.song=list(paste(nxn.dat$note.class,nxn.dat$sub.type,sep="")[which(nxn.dat$sound.files==file.list[i])])
  song.list.fine=append(song.list.fine,new.song)
}
#reorder song lists to match note.count.broad (because this will be the base for song.struct)
song.list.broad=song.list.broad[match(note.count.broad$song.file,file.list)]
song.list.fine=song.list.fine[match(note.count.broad$song.file,file.list)]

#===========================
#COMPUTE AND STORE SONG STRUCTURAL FEATURES
#===========================

#Now, we will compute the structural features for each song, and we will store these in a data frame called song.struct
song.struct=note.count.broad[,1:2]
song.struct$total.notes=rowSums(note.count.broad[,3:ncol(note.count.broad)])
song.struct$unique.notes.b=rowSums(note.count.broad[,3:ncol(note.count.broad)]>0)
song.struct$unique.notes.f=rowSums(note.count.fine[,3:ncol(note.count.fine)]>0)
song.struct$shan.div.b=diversity(note.count.broad[,3:ncol(note.count.broad)],index="shannon")
song.struct$shan.div.f=diversity(note.count.fine[,3:ncol(note.count.fine)],index="shannon")
song.struct$simp.div.b=diversity(note.count.broad[,3:ncol(note.count.broad)],index="simpson")
song.struct$simp.div.f=diversity(note.count.fine[,3:ncol(note.count.fine)],index="simpson")
song.struct$linearity.b=get.linearity(song.list.broad)
song.struct$linearity.f=get.linearity(song.list.fine)
#get the first en block entropies
en=2
for (i in 2:(en+1)){
  song.struct[,(ncol(song.struct)+1):(ncol(song.struct)+2)]=NA
  colnames(song.struct)[(ncol(song.struct)-1):ncol(song.struct)]=c(paste("ent",i-1,"b",sep=""),paste("ent",i-1,"f",sep=""))
  for (j in 1:length(song.list.broad)){
    n.blocks.broad=song.list.broad[[j]][1:(length(song.list.broad[[j]])-i+1)]
    for (k in 2:i){
      n.blocks.broad=paste(n.blocks.broad,song.list.broad[[j]][k:(length(song.list.broad[[j]])-i+k)],sep="")
    }
    p.block.broad=table(n.blocks.broad)/length(n.blocks.broad)
    song.struct[j,(ncol(song.struct)-1)]=sum(-p.block.broad*log(p.block.broad))
    n.blocks.fine=song.list.fine[[j]][1:(length(song.list.fine[[j]])-i+1)]
    for (k in 2:i){
      n.blocks.fine=paste(n.blocks.fine,song.list.fine[[j]][k:(length(song.list.fine[[j]])-i+k)],sep="")
    }
    p.block.fine=table(n.blocks.fine)/length(n.blocks.fine)
    song.struct[j,(ncol(song.struct))]=sum(-p.block.fine*log(p.block.fine))
  }
}
#convert block entropies to differential entropies
if (en>1){
  for (i in (1:2*(en-1))-1){
    song.struct[,ncol(song.struct)-i]=song.struct[,ncol(song.struct)-i]-song.struct[,ncol(song.struct)-i-2]
  }
}
if (en>0){
  song.struct$ent1f=song.struct$ent1f-song.struct$shan.div.f
  song.struct$ent1b=song.struct$ent1b-song.struct$shan.div.b
}
rownames(song.struct)=NULL

#find mean song structural measurements for each bird
song.struct.ic=aggregate(song.struct[,3:ncol(song.struct)],by=list(song.struct$Bird.ID),mean)

#===========================
#CREATE A PEDIGREE WE CAN USE IN REGRESSIONS
#===========================

#convert data to characters
meta.data$Bird.ID=as.character(meta.data$Bird.ID)
meta.data$Genetic.Father=as.character(meta.data$Genetic.Father)
meta.data$Genetic.Mother=as.character(meta.data$Genetic.Mother)
meta.data$GGF_F=as.character(meta.data$GGF_F)
meta.data$GGM_F=as.character(meta.data$GGM_F)
meta.data$GGF_M=as.character(meta.data$GGF_M)
meta.data$GGM_M=as.character(meta.data$GGM_M)

#clean the meta.data to replace unknown parents with NAs
cols.w.ors=grep(" or ",meta.data)
for (i in 1:length(cols.w.ors)){
  meta.data[grep(" or ",meta.data[,cols.w.ors[i]]),cols.w.ors[i]]=NA
}
cols.w.ors=grep("nknown",meta.data)
for (i in 1:length(cols.w.ors)){
  meta.data[grep("nknown",meta.data[,cols.w.ors[i]]),cols.w.ors[i]]=NA
}
recover.meta.data=meta.data

#use information from the grandparents columns to extend the pedigree
male.count=sum(!is.na(unique(c(meta.data$Bird.ID,meta.data$Genetic.Father))))
ped.id=c(meta.data$Bird.ID,unique(meta.data$Genetic.Father[which(!(meta.data$Genetic.Father %in% meta.data$Bird.ID) & !is.na(meta.data$Genetic.Father))]),unique(meta.data$Genetic.Mother[which(!(meta.data$Genetic.Mother %in% meta.data$Bird.ID) & !is.na(meta.data$Genetic.Mother))]))
ped.gf=c(meta.data$Genetic.Father,rep(NA,length(ped.id)-length(meta.data$Genetic.Father)))
ped.gm=c(meta.data$Genetic.Mother,rep(NA,length(ped.id)-length(meta.data$Genetic.Mother)))
#use information about grandparents
for (i in 1:length(ped.id)){
  if (ped.id[i] %in% ped.gf){
    if (is.na(ped.gf[i])){
      ped.gf[i]=as.character(meta.data$GGF_F[match(ped.id[i],meta.data$Genetic.Father)])
    }
    if (is.na(ped.gm[i])){
      ped.gm[i]=as.character(meta.data$GGM_F[match(ped.id[i],meta.data$Genetic.Father)])
    }
  }
  if (ped.id[i] %in% ped.gm){
    if (is.na(ped.gf[i])){
      ped.gf[i]=as.character(meta.data$GGF_M[match(ped.id[i],meta.data$Genetic.Mother)])
    }
    if (is.na(ped.gm[i])){
      ped.gm[i]=as.character(meta.data$GGM_M[match(ped.id[i],meta.data$Genetic.Mother)])
    }
  }
}
#include new additions in ped.id
par.list=unique(c(ped.gf,ped.gm))
par.list=par.list[!is.na(par.list)]
new.ids=setdiff(par.list,ped.id)
new.ids=new.ids[which(new.ids!="Unknown")]
new.sex=rep(2,length(new.ids))
new.sex[which(new.ids %in% ped.gf)]=1
ped.id=c(ped.id,new.ids)
ped.gf=c(ped.gf,rep(NA,length(new.ids)))
ped.gm=c(ped.gm,rep(NA,length(new.ids)))

#create the pedigree
full.ped=pedigree(id=ped.id,dadid=ped.gf,momid=ped.gm,sex=c(rep(1,male.count),rep(2,length(ped.id)-male.count-length(new.sex)),new.sex),affected=0+(ped.id %in% song.struct$Bird.ID),missid="Unknown")
plot.pedigree(full.ped,id=substr(ped.id,4,6),affected=full.ped$affected,cex=.7,align=c(1.5,2))
#build the kinship matrix
kin=kinship(full.ped)
#trim kinship matrix
ID=as.character(song.struct.ic$Group.1)
kin.trim=kin[match(ID,rownames(kin)),match(ID,colnames(kin))]
#create the null kinship matrix
null.kin=diag(nrow(kin.trim))
colnames(null.kin)=colnames(kin.trim)
rownames(null.kin)=rownames(kin.trim)

#========================
#REGRESSIONS FOR STRUCTURAL FEATURES
#========================

#create some predictors we will use in the regressions
sf.in.ssi=meta.data$Social.Father[match(song.struct.ic$Group.1,meta.data$Bird.ID)]
clutch.in.ssi=meta.data$Clutch[match(song.struct.ic$Group.1,meta.data$Bird.ID)]
age=meta.data$Age_Rec[match(song.struct.ic$Group.1,meta.data$Bird.ID)]
log.age=log(age)

#Model selection algorithm

#   1. Select feature to model (by setting col2analyse) and create data objects

#   2. Fit the cultural inheritance model.  Model the value of the song feature in the son as a function of the value of the same feature in the father and the log(age) of the son, with random effects due to clutch and the relatedness matrix. After fitting, we will remove effects of age if they are not at least marginally significant, but we will not remove effects of the social father. If the patterns are real, we should be able to detect social learning with the genetic effect in the model and vice versa. (More generally, the relatedness data enters the model as a random effect, and non-significant random effects should not affect the fitting of fixed effects.)

#   3. Check the residuals of the fitted model. If the residuals are roughly homoskedastic, we can interpret this model. If they fan outward, we can transform them. Log-log transforms will be enough in most cases, although we can try something else if these don't work.

#   4. If necessary, log transform and Winsorise the features

#   5. If necessary, refit the cultural inheritance model and check to make sure the residuals have improved. If they haven't, either try a new transform (code not included) or interpret the model for the untransformed data.

#   6. Fit the null model and compare the cultural inheritance model with pedigree data to the null model. This will give us a p-value for a genetic effect. It does not make sense to estimate heritability or additive genetic variance here, because these will depend on the nest environment, which was imposed by the experiments.

#   7. If there is no genetic effect of the feature, we may wish to refit the mode using lme4 (simply because I trust lme4 more than lmekin - although the underlying code is intended to be similar, and I have not found a case where they give different answers)

#select feature to model, define model structure, and create data vectors
col2analyse=8 #1-13
mod.def="sub.pc~sf.pc+log.age"
mod.def="sub.pc~sf.pc"
colnames(song.struct.ic)[col2analyse+1]
sub.pc=song.struct.ic[,col2analyse+1]
sf.pc=song.struct.ic[match(sf.in.ssi,song.struct.ic$Group.1),col2analyse+1]

# # #log the feature to stabilise variance (if necessary)
#sub.pc=log(song.struct.ic[,col2analyse+1])
#sf.pc=log(song.struct.ic[match(sf.in.ssi,song.struct.ic$Group.1),col2analyse+1])
# #
# # #Windorise in case of -Infs (after logging only)
#sub.pc[which(sub.pc==-Inf)]=min(sub.pc[which(sub.pc!=-Inf)])
#sf.pc[which(sf.pc==-Inf)]=min(sf.pc[which(sf.pc!=-Inf)])

#fit cultural inheritance model 
fmla=as.formula(paste(mod.def,"+(1|ID)+(1|clutch.in.ssi)",sep=""))
pc.lmekin=lmekin(fmla,varlist=2*kin.trim)
pc.lmekin

#fit to full null model
fmla=as.formula(paste(mod.def,"+(1|ID)",sep=""))
pc.lmekin.null.f=lmekin(fmla,varlist=null.kin)
#pc.lmekin.null.f

#fit to null genetics model
fmla=as.formula(paste(mod.def,"+(1|clutch.in.ssi)",sep=""))
pc.lmekin.null.g=lmekin(fmla)
#pc.lmekin.null.g

#fit to null clutch model
fmla=as.formula(paste(mod.def,"+(1|ID)",sep=""))
pc.lmekin.null.c=lmekin(fmla,varlist=2*kin.trim)
#pc.lmekin.null.c

#check residuals (omitting pedigree from model fitting)
fmla=as.formula(paste(mod.def,"+(1|clutch.in.ssi)",sep=""))
mod.4.resids=lmer(fmla)
plot(predict(mod.4.resids),residuals(mod.4.resids))

#highest bar: each significant controlling for the other 
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.g[6][[1]]),1) #tests genetics
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.c[6][[1]]),1) #tests clutch
#at least one significant
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.f[6][[1]]),2)
#significant if the other is clearly not (if one of these is significant and none of the cleaner options above are, we will have to think carefully about what it means)
1-pchisq(2*(pc.lmekin.null.c[6][[1]]-pc.lmekin.null.f[6][[1]]),1) #tests genetics
1-pchisq(2*(pc.lmekin.null.g[6][[1]]-pc.lmekin.null.f[6][[1]]),1) #tests clutch

#===========================
#PREPARE DATA FOR SPECTRAL ANALYSIS
#===========================

#In this section, our goal is to ask whether sons sing particular broad-scale note types in the same ways as their fathers. So, we will begin collecting the average spectral characteristics of each broad-scale note type as produced by each bird. Then, we can study these in mixed regressions
nxn.ic=aggregate(nxn.dat[,c(4:11)],by=list(nxn.dat$note.class,nxn.dat$Bird.ID),mean,na.rm=TRUE)
colnames(nxn.ic)[1:2]=c("note.class","Bird.ID")

#If a bird does not sing a note type at least 5 times, we assume that there is not enough data to work with for that bird and that note, and we exclude those notes from the data set.
nxn.count=aggregate(rep(1,nrow(nxn.dat)),by=list(nxn.dat$note.class,nxn.dat$Bird.ID),sum)
nxn.ic=nxn.ic[which(nxn.count$x>4),]

#The variances in the spectral measures for the individual note types differ by 3 orders of magnitude. 
#aggregate(nxn.ic[,3:ncol(nxn.ic)],by=list(nxn.ic$Group.1),var)
#So, we might want to z-score the values within notes (which will weight note types equally) in the analysis. Thus, we create a z-scored version of nxn.ic.
nxn.z=nxn.ic
note.means=aggregate(nxn.ic[,3:ncol(nxn.ic)],by=list(nxn.ic$note.class),mean,na.rm=TRUE)
note.sds=aggregate(nxn.ic[,3:ncol(nxn.ic)],by=list(nxn.ic$note.class),sd,na.rm=TRUE)
nxn.z[,3:ncol(nxn.z)]=(nxn.z[,3:ncol(nxn.z)]-note.means[match(nxn.z$note.class,note.means$Group.1),2:ncol(note.means)])/note.sds[match(nxn.z$note.class,note.sds$Group.1),2:ncol(note.means)]

#============================
#REGRESSIONS FOR SPECTRAL ANALYSIS
#============================

#Set nxn.use to nxn.ic or nxn.z to study raw or z-scored measures
nxn.use=nxn.z

# # # #remove crazy stacks to test if this weird influential point matters
# nxn.use=nxn.use[-which(nxn.use$Group.1=="Stack"&nxn.use$Group.2=="JS0119"),]

#trim kinship matrices (starting kin created for the sturctural analysis)
ID=as.character(nxn.use$Bird.ID)
individual=as.factor(ID)
kin.trim=kin[match(unique(ID),rownames(kin)),match(unique(ID),colnames(kin))]
#create the null kinship matrix
null.kin=diag(nrow(kin.trim))
colnames(null.kin)=colnames(kin.trim)
rownames(null.kin)=rownames(kin.trim)

#create factors for random effects to be used in the model
sf.in.nxn=meta.data$Social.Father[match(nxn.use$Bird.ID,meta.data$Bird.ID)]
clutch.in.nxn=meta.data$Clutch[match(nxn.use$Bird.ID,meta.data$Bird.ID)]
age=meta.data$Age_Rec[match(nxn.use$Bird.ID,meta.data$Bird.ID)]
log.age=log(age)
cat.name=paste(nxn.use$note.class,nxn.use$Bird.ID,sep="")

#select the feature to analyse (ranging from 1-8)
col2analyse=7
mod.def="sub.pc~sf.pc+log.age"
mod.def="sub.pc~sf.pc"
colnames(nxn.use)[col2analyse+2]
sub.pc=nxn.use[,col2analyse+2]
sf.pc=nxn.use[match(paste(nxn.use$note.class,sf.in.nxn,sep=""),cat.name),col2analyse+2]

#Although we have mean centred within notes, there may be effects of notes if there are effects of the other fixed or random predictors. The attributes of notes are unlikely to be random, because notes differ in particular ways. We can deal with this by forcing the note class into the model as a fixed effect. Because we are not interested in how note types differ, we will not attempt to interpret these fixed effects.  

#full pedigree model
fmla=as.formula(paste(mod.def,"+nxn.use$note.class+(1|ID)+(1|clutch.in.nxn)+(1|individual)",sep=""))
pc.lmekin=lmekin(fmla,varlist=2*kin.trim)
pc.lmekin

#fit to full null model
fmla=as.formula(paste(mod.def,"+nxn.use$note.class+(1|ID)+(1|individual)",sep=""))
pc.lmekin.null.f=lmekin(fmla,varlist=null.kin)
#pc.lmekin.null.f

#fit to null genetics model
fmla=as.formula(paste(mod.def,"+nxn.use$note.class+(1|clutch.in.nxn)+(1|individual)",sep=""))
pc.lmekin.null.g=lmekin(fmla)
#pc.lmekin.null.g

#fit to null clutch model
fmla=as.formula(paste(mod.def,"+nxn.use$note.class+(1|ID)+(1|individual)",sep=""))
pc.lmekin.null.c=lmekin(fmla,varlist=2*kin.trim)
#pc.lmekin.null.c

#check residuals (omitting pedigree from model fitting)
fmla=as.formula(paste(mod.def,"+nxn.use$note.class+(1|clutch.in.nxn)+(1|individual)",sep=""))
mod.4.resids=lmer(fmla)
plot(predict(mod.4.resids),residuals(mod.4.resids))

#highest bar: each significant controlling for the other 
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.g[6][[1]]),1) #tests genetics
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.c[6][[1]]),1) #tests clutch
#at least one significant
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.f[6][[1]]),2)
#significant if the other is clearly not (if one of these is significant and none of the cleaner options above are, we will have to think carefully about what it means)
1-pchisq(2*(pc.lmekin.null.c[6][[1]]-pc.lmekin.null.f[6][[1]]),1) #tests genetics
1-pchisq(2*(pc.lmekin.null.g[6][[1]]-pc.lmekin.null.f[6][[1]]),1) #tests clutch

#===========================
#PREPARE DATA FOR CONSISTENCY ANALYSIS
#===========================

#To analyse consistency, we will ask whether fathers that produce individual note subtypes consistently have sons that also produce individual note subtypes consistently. We will not attempt to match individual note subtypes among birds (as we did for broad note types when we analysed spectral features). Rather, we will assess consistency across all note subtypes together. Thus, we avoid confounding the production of multiple subtypes with inconsistency within subtypes. 

#For many spectral measures, warbleR breaks individual notes into subsections of fixed length for measurement. As a result, these measures are discrete, and assessing variability becomes difficult for very short notes. So, we will exclude problematic features from very short notes from the consistency analysis. (Note that uncertainty about the features of individual notes is less important for the spectral analysis because we are studying the average across all notes of a given class that the bird produces.)

nxn.clean=nxn.dat
#warbleR struggles with time.median and time.IQR for notes with length <0.07 s
nxn.clean$time.median[which(nxn.dat.raw$duration<0.07)]=NA
nxn.clean$time.IQR[which(nxn.dat.raw$duration<0.07)]=NA
#warbeR cannot calculate dfslopes for notes with lengths <0.02 s
nxn.clean$fchange[which(nxn.dat.raw$duration<0.02)]=NA
#warbleR struggles with modindx for notes with lengths <0.05 s
nxn.clean$log.log.modindx[which(nxn.dat.raw$duration<0.05)]=NA

#Collect means, variances, and number of times produced for each note subtype produced in each song.
nxn.c.means=aggregate(nxn.clean[,4:11],by=list(nxn.clean$sound.files,nxn.clean$Bird.ID,nxn.clean$note.class,nxn.clean$sub.type),mean,na.rm=TRUE)
nxn.c.vars=aggregate(nxn.clean[,4:11],by=list(nxn.clean$sound.files,nxn.clean$Bird.ID,nxn.clean$note.class,nxn.clean$sub.type),var,na.rm=TRUE)
nxn.c.tots=aggregate(!is.na(nxn.clean[,4:11]),by=list(nxn.clean$sound.files,nxn.clean$Bird.ID,nxn.clean$note.class,nxn.clean$sub.type),sum)
colnames(nxn.c.means)[1:4]=c("sound.file","Bird.ID","note.class","sub.type")
colnames(nxn.c.vars)[1:4]=c("sound.file","Bird.ID","note.class","sub.type")
colnames(nxn.c.tots)[1:4]=c("sound.file","Bird.ID","note.class","sub.type")

#Compute the weighted mean and variance by note subtype by bird
sub.note.weighted.means=aggregate(nxn.c.means[,5:12]*nxn.c.tots[,5:12],by=list(nxn.c.means$Bird.ID,nxn.c.means$note.class,nxn.c.means$sub.type),sum,na.rm=TRUE)
sub.note.weighted.vars=aggregate(nxn.c.vars[,5:12]*nxn.c.tots[,5:12],by=list(nxn.c.means$Bird.ID,nxn.c.means$note.class,nxn.c.means$sub.type),sum,na.rm=TRUE)
colnames(sub.note.weighted.means)[1:3]=c("Bird.ID","note.class","sub.type")
colnames(sub.note.weighted.vars)[1:3]=c("Bird.ID","note.class","sub.type")
sub.note.tots.m=aggregate(nxn.c.tots[,5:12],by=list(nxn.c.means$Bird.ID,nxn.c.means$note.class,nxn.c.means$sub.type),sum,na.rm=TRUE)
sub.note.weighted.means[,4:11]=sub.note.weighted.means[,4:11]/sub.note.tots.m[,4:11]
sub.note.tots.v=aggregate(nxn.c.tots[,5:12]*(!is.na(nxn.c.vars[,5:12])),by=list(nxn.c.means$Bird.ID,nxn.c.means$note.class,nxn.c.means$sub.type),sum,na.rm=TRUE)
sub.note.weighted.vars[,4:11]=sub.note.weighted.vars[,4:11]/sub.note.tots.v[,4:11]

#We cannot estimate variance accurately unless we have enough observations, so we will exclude variance measures without enough observations
obs.for.var=aggregate(nxn.c.tots[,5:12]-sign(nxn.c.tots[,5:12]),by=list(nxn.c.means$Bird.ID,nxn.c.means$note.class,nxn.c.means$sub.type),sum,na.rm=TRUE)
colnames(obs.for.var)[1:3]=c("Bird.ID","note.class","sub.type")
for (i in 4:11){
  sub.note.weighted.means[which(obs.for.var[,i]<5),i]=NA
  sub.note.weighted.vars[which(obs.for.var[,i]<5),i]=NA
}
#remove empty rows to facilitate examination
sub.note.weighted.vars=sub.note.weighted.vars[which(rowSums(!is.na(sub.note.weighted.means[,4:11]))>0),]
sub.note.tots.m=sub.note.tots.m[which(rowSums(!is.na(sub.note.weighted.means[,4:11]))>0),]
obs.for.var=obs.for.var[which(rowSums(!is.na(sub.note.weighted.means[,4:11]))>0),]
sub.note.weighted.means=sub.note.weighted.means[which(rowSums(!is.na(sub.note.weighted.means[,4:11]))>0),]
#log variances 
sub.note.weighted.vars[,4:11]=log(sub.note.weighted.vars[,4:11])

#For duration, there are three variances with values >9 sds below the mean. We Winsorise these values.
sub.note.weighted.vars[which(sub.note.weighted.vars[,4]<(-15)),4]=min(sub.note.weighted.vars[which(sub.note.weighted.vars[,4]>(-15)),4])

#Create a data frame with just duration, meandom, and fchange for further investigation. 
sub.note.dat=data.frame(sub.note.weighted.means[,c(1:4,7,10)],sub.note.weighted.vars[,c(4,7,10)],sub.note.tots.m[,4])
colnames(sub.note.dat)[4:10]=c("log.duration.m","log.meandom.m","fchange.m","log.duration.v","log.meandom.v","fchange.v","times.produced")
#add the clutch and age
sub.note.dat$clutch=meta.data$Clutch[match(sub.note.dat$Bird.ID,meta.data$Bird.ID)]
sub.note.dat$age=meta.data$Age_Rec[match(sub.note.dat$Bird.ID,meta.data$Bird.ID)]
#add log.dtw, xcor, xcor.count, and xcor.song
load("dtw.RData")
#Winsorise log.dtw
log.dtw$log.dtw[which(log.dtw$log.dtw<(-100))]=min(log.dtw$log.dtw[which(log.dtw$log.dtw>(-100))])
#add the dtw and xcor data to the data set
#NB the Fisher's Z variable is an artefact of an earlier iteration of this code and does not return information with biological meaning, but was left in the code to conserve column numbers etc. 
sub.note.ind=paste(sub.note.dat$Bird.ID,sub.note.dat$note.class,sub.note.dat$sub.type)
consist.ind=paste(log.dtw$Bird.ID,log.dtw$note.class,log.dtw$sub.type)
sub.note.dat$log.dtw=log.dtw$log.dtw[match(sub.note.ind,consist.ind)]
sub.note.dat$xcor=xcor.avg$xcor.avg[match(sub.note.ind,consist.ind)]
sub.note.dat$xcor=xcor.avg$xcor.med[match(sub.note.ind,consist.ind)]
sub.note.dat$fish.z=xcor.avg$fish.z[match(sub.note.ind,consist.ind)]
sub.note.dat$xcor.notes=xcor.scale$x[match(sub.note.ind,consist.ind)]
sub.note.dat$xcor.songs=xcor.in.songs$x[match(sub.note.ind,consist.ind)]
#remove consistency measures based in fewer than 5 comparisons
sub.note.dat[which(sub.note.dat$xcor.notes-sub.note.dat$xcor.songs<5),13:15]=NA


#At this point, we have measures of variability for the different note subtypes produced by each bird. But, these variabilities may depend on the properties of the notes (i.e., duration, meandom, fchange, note class, and number of times produced). We would like to get measures of variability corrected for these predictors. So, we will work with the residuals of a best-fit model. We are missing fchange for 10 note subtypes. Initially, we will fit the models without these data points included. If they are not important, we can refit models with all of the data, but excluding the predictors with missing values. 

#exclude Chips, because our Gaussian mixture models did not produce reliable clusters that would represent note subtypes
obs.for.var=obs.for.var[-which(sub.note.dat$note.class=="Chip"),]
sub.note.dat=sub.note.dat[-which(sub.note.dat$note.class=="Chip"),]

sub.note.reduced=sub.note.dat[which(!is.na(sub.note.dat$age)&!is.na(sub.note.dat$fchange.m)),]
obs.for.var.reduced=obs.for.var[which(!is.na(sub.note.dat$age)&!is.na(sub.note.dat$fchange.m)),c(4,7,10)]
obs.for.var.reduced$dtw=sub.note.reduced$xcor.notes-sub.note.reduced$xcor.songs
obs.for.var.reduced$xcor=obs.for.var.reduced$dtw
obs.for.var.reduced$fish.z=obs.for.var.reduced$dtw

#make interactions. We will add interaction terms to our models explicitly, because this makes it easier to collect the regression coefficients
dur.x.cha=sub.note.reduced$log.duration.m*sub.note.reduced$fchange.m
dur.x.mea=sub.note.reduced$log.duration.m*sub.note.reduced$log.meandom.m
mea.x.cha=sub.note.reduced$log.meandom.m*sub.note.reduced$fchange.m
x3=sub.note.reduced$log.duration.m*sub.note.reduced$log.meandom.m*sub.note.reduced$fchange.m

#find best models w/o age (by brute force)
resp.vs=c("log.duration.v","log.meandom.v","fchange.v","log.dtw","xcor","fish.z")
mod.tags=c("dur","mea","cha","dtw","xcor","fish")
mod.count=0
for (modloop in 1:6){
  resp.v=resp.vs[modloop]
  predictors=c("log.duration.m","fchange.m","log.meandom.m","dur.x.cha","dur.x.mea","mea.x.cha","I(log.duration.m^2)","I(fchange.m^2)","I(log.meandom.m^2)","x3","note.class","I(log(times.produced))")
  if (modloop==4){
    predictors=c("log.duration.m","fchange.m","log.meandom.m","dur.x.cha","dur.x.mea","mea.x.cha","I(log.duration.m^2)","I(fchange.m^2)","I(log.meandom.m^2)","x3","note.class","I(log(xcor.notes))")
  }
  if (modloop>4){
    predictors=c("log.duration.m","fchange.m","log.meandom.m","dur.x.cha","dur.x.mea","mea.x.cha","I(log.duration.m^2)","I(fchange.m^2)","I(log.meandom.m^2)","x3","note.class","I(log(xcor.notes))","I(log(xcor.songs))")
  }
  weight.tab=rep(0,2+length(predictors)+length(unique(sub.note.reduced$note.class)))
  weight.tab=as.data.frame(rbind(weight.tab,weight.tab))
  colnames(weight.tab)=c("AICc","(Intercept)",predictors,paste("note.class",unique(sub.note.reduced$note.class),sep=""))
  weight.tab=weight.tab[-2,]
  best.mod=c("1")
  fmla=as.formula(paste(resp.v,"~1+(1|Bird.ID)+(1|clutch)"))
  m.lm=lmer(fmla,weights=sqrt(obs.for.var.reduced[,modloop]),data=sub.note.reduced,REML=FALSE)
  #best.aic=AICc(m.lm)
  best.aic=useBIC(m.lm)
  weight.tab[1]=best.aic
  for (numin in 1:length(predictors)){
    compred=combn(predictors,numin)
    #clean out unnecessary models (for speed)
    if (numin>1 & numin<length(predictors)){
      c2cut=(colSums(compred=="log.duration.m")==0 & (colSums(compred=="dur.x.cha")>0 | colSums(compred=="dur.x.mea")>0 | colSums(compred=="I(log.duration.m^2)")>0))
      compred=compred[,-which(c2cut)]
      c2cut=(colSums(compred=="log.meandom.m")==0 & (colSums(compred=="mea.x.cha")>0 | colSums(compred=="dur.x.mea")>0 | colSums(compred=="I(log.meandom.m^2)")>0))
      compred=compred[,-which(c2cut)]
      c2cut=(colSums(compred=="fchange.m")==0 & (colSums(compred=="mea.x.cha")>0 | colSums(compred=="dur.x.cha")>0 | colSums(compred=="I(fchange.m^2)")>0))
      compred=compred[,-which(c2cut)]
      c2cut=(colSums(compred=="x3")>0 & (colSums(compred=="mea.x.cha")==0 | colSums(compred=="dur.x.cha")==0 | colSums(compred=="dur.x.mea")==0))
      compred=compred[,-which(c2cut)]
  }
    
    for (icol in 1:ncol(compred)){
      mod.count=mod.count+1
      if (mod.count%%100==0){
        print(mod.count)
      }
      fmla=as.formula(paste(resp.v,"~",paste(compred[,icol], collapse= "+"),"+(1|Bird.ID)+(1|clutch)"))
      m.lm=lmer(fmla,weights=sqrt(obs.for.var.reduced[,modloop]),data=sub.note.reduced,REML=FALSE)
      
      #new.AIC=AICc(m.lm)
      new.AIC=useBIC(m.lm)
      if (new.AIC<best.aic){
        best.mod=fmla
        best.aic=new.AIC
      }
      #store cAIC and coefficients to get weighted coeffs later
      weight.tab=rbind(weight.tab,rep(0,2+length(predictors)+length(unique(sub.note.reduced$note.class))))
      weight.tab[nrow(weight.tab),1]=new.AIC
      cfs=coef(m.lm)[[1]][1,]
      weight.tab[nrow(weight.tab),match(colnames(cfs),colnames(weight.tab))]=cfs
    }
  }
  m.lm=lmer(best.mod,weights=sqrt(obs.for.var.reduced[,modloop]),data=sub.note.reduced)
  assign(paste("best.mod.na.",mod.tags[modloop],sep=""),m.lm)
  #get weighted coefs
  weight.tab[,1]=weight.tab[,1]-min(weight.tab[,1])
  Akaike.weight=exp(-weight.tab[,1]/2)
  Akaike.weight=Akaike.weight/sum(Akaike.weight)
  weighted.coefs=colSums(weight.tab[,2:ncol(weight.tab)]*Akaike.weight)
  assign(paste("weighted.coefs.",mod.tags[modloop],sep=""),weighted.coefs)
}  

#Consistency increases with times produced for all three predictors. We will estimate ages for the six birds with missing ages. This will allow us to estimate their residual consistency (after controlling for note features and age) to use as a response variable, and then to estimate the observed consistency and residual consistency for social fathers when they are raising sons through recursive fitting. 

#Birth dates correlate closely with bird numbers, so I will assign birth dates to match the bird with the most similar number.
meta.data=recover.meta.data
meta.data$Birth.Date=as.character(meta.data$Birth.Date)
meta.data$Birth.Date[which(meta.data$Bird.ID=="JS0002")]="15/09/2007"
meta.data$Birth.Date[which(meta.data$Bird.ID=="JS0005")]="15/09/2007"  #bird not in pedigree or social father
meta.data$Birth.Date[which(meta.data$Bird.ID=="JS0036")]="01/07/2009"
meta.data$Birth.Date[which(meta.data$Bird.ID=="JS0064")]="12/12/2010"  #was father on 20/08/2011
meta.data$Birth.Date[which(meta.data$Bird.ID=="JS0119")]="25/12/2010"
meta.data$Birth.Date[which(meta.data$Bird.ID=="JS0155")]="06/10/2010"
for (i in 1:length(meta.data$Birth.Date)){
  if (substr(meta.data$Birth.Date[i],1,2)=="x/"){
    meta.data$Birth.Date[i]=paste("15",substr(meta.data$Birth.Date[i],2,nchar(meta.data$Birth.Date[i])),sep="")
  }
  if (substr(meta.data$Birth.Date[i],3,5)=="/x/"){
    meta.data$Birth.Date[i]=paste("01/07",substr(meta.data$Birth.Date[i],5,nchar(meta.data$Birth.Date[i])),sep="")
  }
  if (nchar(meta.data$Birth.Date[i])>10){
    meta.data$Birth.Date[i]=substr(meta.data$Birth.Date[i],1,10)
  }
}
meta.data$Birth.Date=as.Date(meta.data$Birth.Date,"%d/%m/%Y")
meta.data$DOR=as.Date(meta.data$DOR,"%d/%m/%Y")

#add missing ages
meta.data$Age_Rec=as.numeric(meta.data$DOR-meta.data$Birth.Date)/365.25

#correct for note effects
sub.note.dat$age=meta.data$Age_Rec[match(sub.note.dat$Bird.ID,meta.data$Bird.ID)]
#remove note effects
sub.note.dat$dur.resid=sub.note.dat$log.duration.v-weighted.coefs.dur[match(paste("note.class",sub.note.dat$note.class,sep=""),names(weighted.coefs.dur))]
sub.note.dat$mea.resid=sub.note.dat$log.meandom.v-weighted.coefs.mea[match(paste("note.class",sub.note.dat$note.class,sep=""),names(weighted.coefs.mea))]
sub.note.dat$cha.resid=sub.note.dat$fchange.v-weighted.coefs.cha[match(paste("note.class",sub.note.dat$note.class,sep=""),names(weighted.coefs.cha))]
sub.note.dat$dtw.resid=sub.note.dat$log.dtw-weighted.coefs.dtw[match(paste("note.class",sub.note.dat$note.class,sep=""),names(weighted.coefs.dtw))]
sub.note.dat$xcor.resid=sub.note.dat$xcor-weighted.coefs.xcor[match(paste("note.class",sub.note.dat$note.class,sep=""),names(weighted.coefs.xcor))]
sub.note.dat$fish.resid=sub.note.dat$fish.z-weighted.coefs.fish[match(paste("note.class",sub.note.dat$note.class,sep=""),names(weighted.coefs.fish))]
#remove note attribute effects
sub.note.dat$dur.resid=sub.note.dat$dur.resid+rowSums(-t(t(sub.note.dat[,c(4:6)])*weighted.coefs.dur[c(2,4,3)])-t(t(sub.note.dat[,c(4:6)]^2)*weighted.coefs.dur[c(8,10,9)])-t(t(sub.note.dat[,c(4:6)]*sub.note.dat[,c(6,4,5)])*weighted.coefs.dur[c(5,6,7)]))-(sub.note.dat[,4]*sub.note.dat[,5]*sub.note.dat[,6]*weighted.coefs.dur[11])
sub.note.dat$mea.resid=sub.note.dat$mea.resid+rowSums(-t(t(sub.note.dat[,c(4:6)])*weighted.coefs.mea[c(2,4,3)])-t(t(sub.note.dat[,c(4:6)]^2)*weighted.coefs.mea[c(8,10,9)])-t(t(sub.note.dat[,c(4:6)]*sub.note.dat[,c(6,4,5)])*weighted.coefs.mea[c(5,6,7)]))-(sub.note.dat[,4]*sub.note.dat[,5]*sub.note.dat[,6]*weighted.coefs.mea[11])
sub.note.dat$cha.resid=sub.note.dat$cha.resid+rowSums(-t(t(sub.note.dat[,c(4:6)])*weighted.coefs.cha[c(2,4,3)])-t(t(sub.note.dat[,c(4:6)]^2)*weighted.coefs.cha[c(8,10,9)])-t(t(sub.note.dat[,c(4:6)]*sub.note.dat[,c(6,4,5)])*weighted.coefs.cha[c(5,6,7)]))-(sub.note.dat[,4]*sub.note.dat[,5]*sub.note.dat[,6]*weighted.coefs.cha[11])
sub.note.dat$dtw.resid=sub.note.dat$dtw.resid+rowSums(-t(t(sub.note.dat[,c(4:6)])*weighted.coefs.dtw[c(2,4,3)])-t(t(sub.note.dat[,c(4:6)]^2)*weighted.coefs.dtw[c(8,10,9)])-t(t(sub.note.dat[,c(4:6)]*sub.note.dat[,c(6,4,5)])*weighted.coefs.dtw[c(5,6,7)]))-(sub.note.dat[,4]*sub.note.dat[,5]*sub.note.dat[,6]*weighted.coefs.dtw[11])
sub.note.dat$xcor.resid=sub.note.dat$xcor.resid+rowSums(-t(t(sub.note.dat[,c(4:6)])*weighted.coefs.xcor[c(2,4,3)])-t(t(sub.note.dat[,c(4:6)]^2)*weighted.coefs.xcor[c(8,10,9)])-t(t(sub.note.dat[,c(4:6)]*sub.note.dat[,c(6,4,5)])*weighted.coefs.xcor[c(5,6,7)]))-(sub.note.dat[,4]*sub.note.dat[,5]*sub.note.dat[,6]*weighted.coefs.xcor[11])
sub.note.dat$fish.resid=sub.note.dat$fish.resid+rowSums(-t(t(sub.note.dat[,c(4:6)])*weighted.coefs.fish[c(2,4,3)])-t(t(sub.note.dat[,c(4:6)]^2)*weighted.coefs.fish[c(8,10,9)])-t(t(sub.note.dat[,c(4:6)]*sub.note.dat[,c(6,4,5)])*weighted.coefs.fish[c(5,6,7)]))-(sub.note.dat[,4]*sub.note.dat[,5]*sub.note.dat[,6]*weighted.coefs.fish[11])
#remove effects of times produced
sub.note.dat$dur.resid=sub.note.dat$dur.resid-log(sub.note.dat[,10])*weighted.coefs.dur[13]
sub.note.dat$cha.resid=sub.note.dat$cha.resid-log(sub.note.dat[,10])*weighted.coefs.cha[13]
sub.note.dat$mea.resid=sub.note.dat$mea.resid-log(sub.note.dat[,10])*weighted.coefs.mea[13]
sub.note.dat$dtw.resid=sub.note.dat$dtw.resid-log(sub.note.dat[,16])*weighted.coefs.dtw[13]
sub.note.dat$xcor.resid=sub.note.dat$xcor.resid-log(sub.note.dat[,16])*weighted.coefs.xcor[13]-log(sub.note.dat[,17])*weighted.coefs.xcor[14]
sub.note.dat$fish.resid=sub.note.dat$fish.resid-log(sub.note.dat[,16])*weighted.coefs.fish[13]-log(sub.note.dat[,17])*weighted.coefs.fish[14]
  
#examine correlations
cor(sub.note.dat[,c(7:9,13:15)],use="complete.obs")
cor(sub.note.dat[,c(18:23)],use="complete.obs")

#Get mean consistency by bird. We will weight every note type equally, regardless of how many times the bird produces it. We note thought that there does not appear to be a correlation between the number of times a note was produced and its residual variability.
sub.note.consist=aggregate(sub.note.dat[,18:23],by=list(sub.note.dat$Bird.ID),mean,na.rm=TRUE)
colnames(sub.note.consist)[1]="Bird.ID"
sub.note.consist.obs=aggregate(sub.note.dat[,c(7:9,13:15)],by=list(sub.note.dat$Bird.ID),mean,na.rm=TRUE)
colnames(sub.note.consist.obs)[1]="Bird.ID"

#===========================
#REGRESSIONS FOR CONSISTENCY ANALYSIS
#===========================

#trim kinship matrices (starting kin created for the sturctural analysis)
ID=as.character(sub.note.consist$Bird.ID)
individual=as.factor(ID)
kin.trim=kin[match(unique(ID),rownames(kin)),match(unique(ID),colnames(kin))]
#create the null kinship matrix
null.kin=diag(nrow(kin.trim))
colnames(null.kin)=colnames(kin.trim)
rownames(null.kin)=rownames(kin.trim)

#create factors for random effects to be used in the model
sf.in.con=meta.data$Social.Father[match(sub.note.consist$Bird.ID,meta.data$Bird.ID)]
clutch.in.con=meta.data$Clutch[match(sub.note.consist$Bird.ID,meta.data$Bird.ID)]
#get ages of focal birds
sub.age=meta.data$Age_Rec[match(sub.note.consist$Bird.ID,meta.data$Bird.ID)]
sf.age.at.son=as.numeric(meta.data$Birth.Date[match(sub.note.consist$Bird.ID,meta.data$Bird.ID)]-meta.data$Birth.Date[match(sf.in.con,meta.data$Bird.ID)])/365.25
sf.age.at.rec=as.numeric(meta.data$DOR[match(sf.in.con,meta.data$Bird.ID)]-meta.data$Birth.Date[match(sf.in.con,meta.data$Bird.ID)])/365.25

#select the feature to analyse (ranging from 1-5)
col2analyse=4
mod.def="sub.pc~sf.resid+I(log(sub.age))"
mod.def="sub.pc~sf.resid"
#mod.def="sub.pc~sf.obs+I(log(sub.age))"  ###not used
#mod.def="sub.pc~sf.obs"                  ###not used
colnames(sub.note.consist)[col2analyse+1]
sub.pc=sub.note.consist[,col2analyse+1]
sf.resid.raw=sub.note.consist[match(sf.in.con,sub.note.consist$Bird.ID),col2analyse+1]
sf.obs.raw=sub.note.consist.obs[match(sf.in.con,sub.note.consist.obs$Bird.ID),col2analyse+1]

#Correct father's songs for age with recursive fitting. Here, we fit the model using the father's consistencies without age correction, then correct using the effect of age estimated from the model, and refit. We repeat until the effects of age in the model and the correction converge.
if (grepl("age",mod.def)){
  #fit model to estimate age 
  fmla=as.formula(paste(mod.def,"+(1|ID)+(1|clutch.in.con)",sep=""))
  sf.resid=sf.resid.raw
  sf.obs=sf.obs.raw
  age.eff=0
  stopper=1
  while (abs(stopper)>10^(-6)){
    fit.4.age=lmekin(fmla,varlist=2*kin.trim)
    print((age.eff-fixef(fit.4.age)[[length(fixef(fit.4.age))]])/(fixef(fit.4.age)[[length(fixef(fit.4.age))]]))
    stopper=((age.eff-fixef(fit.4.age)[[length(fixef(fit.4.age))]])/(fixef(fit.4.age)[[length(fixef(fit.4.age))]]))
    age.eff=fixef(fit.4.age)[[length(fixef(fit.4.age))]]
    sf.resid=sf.resid.raw+log(sf.age.at.son)*age.eff
    sf.obs=sf.obs.raw-age.eff*log(sf.age.at.rec)+age.eff*log(sf.age.at.son)
  }
}

#full pedigree model
fmla=as.formula(paste(mod.def,"+(1|ID)+(1|clutch.in.con)",sep=""))
pc.lmekin=lmekin(fmla,varlist=2*kin.trim)
pc.lmekin

#fit to full null model
fmla=as.formula(paste(mod.def,"+(1|ID)",sep=""))
pc.lmekin.null.f=lmekin(fmla,varlist=null.kin)
#pc.lmekin.null.f

#fit to null genetics model
fmla=as.formula(paste(mod.def,"+(1|clutch.in.con)",sep=""))
pc.lmekin.null.g=lmekin(fmla)
#pc.lmekin.null.g

#fit to null clutch model
fmla=as.formula(paste(mod.def,"+(1|ID)",sep=""))
pc.lmekin.null.c=lmekin(fmla,varlist=2*kin.trim)
#pc.lmekin.null.c

#check residuals (omitting pedigree from model fitting)
mod.4.resids=lm(as.formula(mod.def))
plot(predict(mod.4.resids),residuals(mod.4.resids))
qqnorm(residuals(mod.4.resids))

#highest bar: each significant controlling for the other 
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.g[6][[1]]),1) #tests genetics
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.c[6][[1]]),1) #tests clutch
#at least one significant
1-pchisq(2*(pc.lmekin[6][[1]]-pc.lmekin.null.f[6][[1]]),2)
# #significant if the other is clearly not (if one of these is significant and none of the cleaner options above are, we will have to think carefully about what it means)
1-pchisq(2*(pc.lmekin.null.c[6][[1]]-pc.lmekin.null.f[6][[1]]),1) #tests genetics
1-pchisq(2*(pc.lmekin.null.g[6][[1]]-pc.lmekin.null.f[6][[1]]),1) #tests clutch

